package iteration;

/** Quelques programmes pour commencer.
 */
public abstract class TP1 {
  /** Constructeur privé.
   * Ne devrait pas être appelé.
   */
  private TP1() {}

  /** Programme vide.
   * @return une « itération » triviale
   */
  public static Iteration noop() {
    return new Iteration() {
      public boolean isDone() { return true; }
      public Iteration step() { return this; }
      public String toString() { return "noop"; }
    };
  }

  /** Bonjour le monde.
   * Affiche un message de bienvenue.
   * @return le fameux « Hello World! »
   */
  public static Iteration helloWorld() {
    return new Iteration() {
      {
	// Ce code est exécuté à la création de l’objet
	System.out.println("Bonjour le monde.");
      }
      public boolean isDone() { return true; }
      public Iteration step() { return this; }
      public String toString() { return "helloWorld"; }
    };
  }

  /** Suite de Fibonacci.
   * Ce programme ne termine pas et
   * calcule les termes de la suite de Fibonacci,
   * en arithmétique machine signée 32 bits.
   */
  public static Iteration fibo() {
    return new Iteration() {
      int curr = 1;
      int next = 1;
      public boolean isDone() { return false; }
      public Iteration step() {
	int prev = curr;
	curr = next;
	next = curr + prev;
	return this;
      }
      public String toString() {
	return String.format("%d", curr);
      }
    };
  }

  /** Lecture dans un tableau avec valeur par défaut.
   * @return t[n] ou def
   */
  static private <T> T get(final T[] t, final int n, final T def) {
    try {
      return t[n];
    } catch (ArrayIndexOutOfBoundsException e) {
      return def;
    }
  }

  /** Choix du programme selon les arguments de la ligne de commande.
   */
  static Iteration select(final String[] args) {
    final String p = get(args, 0, "");
    if ("hw".equals(p)) return helloWorld();
    if ("fibo".equals(p)) return fibo();
    return noop();
  }

  /** Fonction principale.
   * Construit une instance d’{@link Iteration} selon les arguments de la
   * ligne de commande, puis l’exécute :
   * <dl>
   * <dt>hw</dt><dd>{@link #helloWorld}</dd>
   * <dt>fibo</dt><dd>{@link #fibo}</dd>
   * </dl>
   */
  public static void main(final String[] args) {
    final Iteration i = select(args);
    while ( ! i.isDone() ) {
      System.out.println(String.format("%s", i));
      i.step();
    }
    System.out.println(String.format("État final: %s", i));
  }
}
